const semver = require('semver');
const engines = require('./package.json').engines;
const path = require('path');
require('source-map-support').install();

let controller;
let stopping = false;

async function restart() {
    await stop(true);
    await start();
}

async function exit(code, restart) {
    if (!restart) {
        process.exit(code);
    }
}

async function start() {
    const version = engines.node;
    if (!semver.satisfies(process.version, version)) {
        console.log(`\t\tZigbee2MQTT requires node version ${version}, you are running ${process.version}!\n`); // eslint-disable-line
    }

    // Validate settings
    const settings = require('./dist/util/settings');
    settings.reRead();
    const errors = settings.validate();
    if (errors.length > 0) {
        console.log(`\n\n!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!`);
        console.log('            READ THIS CAREFULLY\n');
        console.log(`Refusing to start because configuration is not valid, found the following errors:`);
        for (const error of errors) {
            console.log(`- ${error}`);
        }
        console.log(`\nIf you don't know how to solve this, read https://www.zigbee2mqtt.io/guide/configuration`); // eslint-disable-line
        console.log(`\n!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n\n`);
        exit(1);
    }

    const Controller = require('./dist/controller');
    controller = new Controller(restart, exit);
    await controller.start();
}

async function stop(restart) {
    await controller.stop(restart);
}

async function handleQuit() {
    if (!stopping && controller) {
        stopping = true;
        await stop(false);
    }
}

if (require.main === module || require.main.filename.endsWith(path.sep + 'cli.js')) {
    process.on('SIGINT', handleQuit);
    process.on('SIGTERM', handleQuit);
    start();
} else {
    process.on('SIGINT', handleQuit);
    process.on('SIGTERM', handleQuit);
    module.exports = {start};
}
